@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2008  Michel de Boer <michel@twinklephone.com>
+    Copyright (C) 2005-2009  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -367,7 +367,7 @@ void t_call_history::add_call_record(const t_call_record &call_record, bool writ
 
 	records.push_back(call_record);
 	
-	while (records.size() > sys_config->get_ch_max_size()) {
+	while (records.size() > (size_t)sys_config->get_ch_max_size()) {
 		records.pop_front();
 	}
 	
@@ -417,112 +417,6 @@ void t_call_history::delete_call_record(unsigned short id, bool write) {
 	ui->cb_call_history_updated();
 }
 
-/*
-bool t_call_history::read_history(string &error_msg) {
-	struct stat stat_buf;
-	
-	mtx_records.lock();
-	
-	records.clear();
-	
-	// Check if call history file exists
-	if (stat(filename.c_str(), &stat_buf) != 0) {
-		// There is no call history file.
-		mtx_records.unlock();
-		return true;
-	}
-	
-	// Open call history file
-	ifstream ch(filename.c_str());
-	if (!ch) {
-		error_msg = TRANSLATE("Cannot open file for reading: %1");
-		error_msg = replace_first(error_msg, "%1", filename);
-		mtx_records.unlock();
-		return false;
-	}
-	
-	// Read and parse history file.
-	while (!ch.eof()) {
-		string line;
-		t_call_record rec;
-		
-		getline(ch, line);
-
-		// Check if read operation succeeded
-		if (!ch.good() && !ch.eof()) {
-			error_msg = TRANSLATE("File system error while reading file %1 .");
-			error_msg = replace_first(error_msg, "%1", filename);
-			mtx_records.unlock();
-			return false;
-		}
-
-		line = trim(line);
-
-		// Skip empty lines
-		if (line.size() == 0) continue;
-
-		// Skip comment lines
-		if (line[0] == '#') continue;
-		
-		// Add record. Skip records that cannot be parsed.
-		if (rec.populate_from_file_record(line)) {
-			add_call_record(rec, false);
-		}
-	}
-	
-	mtx_records.unlock();
-	
-	// Clear the number of missed calls as reading the history
-	// will have increased the number of missed calls for each
-	// record read.
-	clear_num_missed_calls();
-	return true;
-}
-
-bool t_call_history::write_history(string &error_msg) const {
-	struct stat stat_buf;
-	
-	t_call_history *self = const_cast<t_call_history *>(this);
-	
-	self->mtx_records.lock();
-	
-	// Open file
-	ofstream ch(filename.c_str());
-	if (!ch) {
-		error_msg = TRANSLATE("Cannot open file for writing: %1");
-		error_msg = replace_first(error_msg, "%1", filename);
-		self->mtx_records.unlock();
-		return false;
-	}
-	
-	// Write file header
-	ch << "# time_start|time_answer|time_end|direction|from_display|from_uri|"
-	      "from_organization|to_display|to_uri|to_organization|"
-	      "reply_to_display|reply_to_uri|referred_by_display|referred_by_uri|"
-	      "subject|rel_cause|invite_resp_code|invite_resp_reason|"
-	      "far_end_device|user_profile";
-	ch << endl;
-	      
-	// Write records
-	for (list<t_call_record>::const_iterator i = records.begin();
-	     i != records.end(); i++)
-	{
-		ch << i->create_file_record();
-		ch << endl;
-	}
-	
-	self->mtx_records.unlock();
-	 
-	if (!ch.good()) {
-		error_msg = TRANSLATE("File system error while writing file %1 .");
-		error_msg = replace_first(error_msg, "%1", filename);
-		return false;
-	}
-	
-	return true;
-}
-*/
-
 void t_call_history::get_history(list<t_call_record> &history) {
 	mtx_records.lock();
 	history = records;
