@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2008  Michel de Boer <michel@twinklephone.com>
+    Copyright (C) 2005-2009  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include "audio/audio_codecs.h"
 #include "parser/sip_message.h"
 #include "sockets/socket.h"
+#include "sockets/url.h"
 #include "threads/mutex.h"
 #include "threads/sema.h"
 
@@ -55,6 +56,8 @@ enum t_event_type {
 	EV_ICMP,		/**< ICMP error */
 	EV_UI,			/**< User interface event */
 	EV_ASYNC_RESPONSE,	/**< Response on an asynchronous question */
+	EV_BROKEN_CONNECTION,	/**< Persitent connection to SIP proxy broken */
+	EV_TCP_PING,		/**< Send a TCP ping (double CRLF) */
 };
 
 /** Abstract parent class for all events */
@@ -459,6 +462,7 @@ public:
 
 /** User interface callback types. */
 enum t_ui_event_type {
+	TYPE_UI_CB_DISPLAY_MSG,			/**< Display a message */
 	TYPE_UI_CB_DTMF_DETECTED,		/**< DTMF tone detected */
 	TYPE_UI_CB_SEND_DTMF,			/**< Sending DTMF */
 	TYPE_UI_CB_RECV_CODEC_CHANGED,		/**< Codec changed */
@@ -469,6 +473,14 @@ enum t_ui_event_type {
 	TYPE_UI_CB_QUIT				/**< Quit the user interface */
 };
 
+/** Display message priorities. */
+enum t_msg_priority {
+	MSG_NO_PRIO,
+	MSG_INFO,
+	MSG_WARNING,
+	MSG_CRITICAL
+};
+
 /**
  * User interface event.
  * Send a user interface callback to the user interface.
@@ -488,6 +500,8 @@ private:
 	bool		encrypted;	/**< Encryption indication. */
 	string		cipher_mode;	/**< Cipher mode (algorithm name). */
 	string		zrtp_sas;	/**< ZRTP SAS/ */
+	t_msg_priority	msg_priority;	/**< Priority of a display message. */
+	string		msg;		/**> Message to display. */
 	//@}
 
 public:
@@ -507,6 +521,7 @@ public:
 	void set_encrypted(bool on);
 	void set_cipher_mode(const string &_cipher_mode);
 	void set_zrtp_sas(const string &sas);
+	void set_display_msg(const string &_msg, t_msg_priority &_msg_priority);
 	//@}
 	
 	/**
@@ -562,6 +577,56 @@ public:
 	bool get_bool_response(void) const;
 };
 
+/**
+ * Broken connection event.
+ * A persistent connection to a SIP proxy is broken. With this event
+ * the transport layer signals the transaction layer that a connection
+ * is broken.
+ */
+class t_event_broken_connection : public t_event {
+private:
+	/** The user URI (AoR) that the connection was associated with. */
+	t_url		user_uri_;
+	
+public:
+	/** Constructor */
+	t_event_broken_connection(const t_url &url);
+	
+	t_event_type get_type(void) const;
+	
+	/**
+	 * Get the user URI.
+	 * @return The user URI.
+	 */
+	t_url get_user_uri(void) const;
+};
+
+/**
+ * TCP ping event.
+ * Send a TCP ping (double CRLF).
+ */
+class t_event_tcp_ping : public t_event {
+private:
+	/** The user URI (AoR) for which the ping must be sent. */
+	t_url		user_uri_;
+	
+	unsigned int	dst_addr_;	/**< Destination address for ping (host order) */
+	unsigned short	dst_port_;	/**< Destination port (host order) */
+	
+public:
+	/** Constructor */
+	t_event_tcp_ping(const t_url &url, unsigned int dst_addr, unsigned short dst_port);
+	
+	t_event_type get_type(void) const;
+	
+	/** @name Getters */
+	//@{
+	t_url get_user_uri(void) const;
+	unsigned int get_dst_addr(void) const;
+	unsigned short get_dst_port(void) const;
+	//@}
+};
+
 
 /**
  * Event queue.
@@ -728,6 +793,20 @@ public:
 	 * @param permission [in] Permission allowed?.
 	 */
 	void push_refer_permission_response(bool permission);
+	
+	/**
+	 * Create a broken connection event.
+	 * @param user_uri [in] The user URI (AoR) associated with the connection.
+	 */
+	void push_broken_connection(const t_url &user_uri);
+	
+	/**
+	 * Create a TCP ping event.
+	 * @param user_uri [in] The user URI (AoR) for which the TCP ping must be sent.
+	 * @param dst_addr [in] The destination IPv4 address for the ping.
+	 * @param dst_port [in] The destination TCP port for the ping.
+	 */
+	void push_tcp_ping(const t_url &user_uri, unsigned int dst_addr, unsigned short dst_port);
 
 	/**
 	 * Pop an event from the queue. 
