@@ -7,7 +7,7 @@
 ** place of a destructor.
 *****************************************************************************/
 /*
-    Copyright (C) 2005-2008  Michel de Boer <michel@twinklephone.com>
+    Copyright (C) 2005-2009  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@ void GetProfileNameForm::validate()
 	}
 	
 	QString filename = profileLineEdit->text();
-	filename.append(".cfg");
+	filename.append(USER_FILE_EXT);
 	QString fullname = d.filePath(filename);
 	if (QFile::exists(fullname)) {
 		QMessageBox::warning(this, PRODUCT_NAME,
