@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2008  Michel de Boer <michel@twinklephone.com>
+    Copyright (C) 2005-2009  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -151,6 +151,8 @@ void t_log::write_header(const string &func_name, t_log_class log_class,
 			return;
 		}
 		break;
+	default:
+		break;
 	}
 
 	struct timeval t;
@@ -285,9 +287,9 @@ void t_log::write_raw(const string &raw) {
 	if (log_disabled || log_report_disabled) return;
 	
 	if (raw.size() < MAX_LEN_LOG_STRING) {
-		*log_stream << raw;
+		*log_stream << to_printable(raw);
 	} else {
-		*log_stream << raw.substr(0, MAX_LEN_LOG_STRING);
+		*log_stream << to_printable(raw.substr(0, MAX_LEN_LOG_STRING));
 		*log_stream << "\n\n";
 		*log_stream << "<cut off>\n";
 	}
