@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2008  Michel de Boer <michel@twinklephone.com>
+    Copyright (C) 2005-2009  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,18 +18,28 @@
 
 #include "twinkle_config.h"
 
+
+#include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/soundcard.h>
+#include <dirent.h>
 #include <unistd.h>
 #include <fstream>
 #include <iostream>
 #include <cstring>
+
 #include "sys_settings.h"
+
+#include "log.h"
 #include "translator.h"
 #include "user.h"
 #include "userintf.h"
 #include "util.h"
+#include "audits/memman.h"
+#include "utils/file_utils.h"
+
+using namespace utils;
 
 // Share directory containing files applicable to all users
 #define DIR_SHARE	DATADIR
@@ -40,13 +50,15 @@
 // System config file
 #define SYS_CONFIG_FILE	"twinkle.sys"
 
+// Default location of the shared mime database
+#define DFLT_SHARED_MIME_DB	"/usr/share/mime/globs"
+
 // Field names in the config file
 // AUDIO fields
 #define FLD_DEV_RINGTONE	"dev_ringtone"
 #define FLD_DEV_SPEAKER		"dev_speaker"
 #define FLD_DEV_MIC		"dev_mic"
 #define FLD_VALIDATE_AUDIO_DEV	"validate_audio_dev"
-#define FLD_AU_REDUCE_NOISE_MIC	"au_reduce_noise_mic"
 #define FLD_ALSA_PLAY_PERIOD_SIZE	"alsa_play_period_size"
 #define FLD_ALSA_CAPTURE_PERIOD_SIZE	"alsa_capture_period_size"
 #define FLD_OSS_FRAGMENT_SIZE	"oss_fragment_size"
@@ -63,6 +75,7 @@
 #define FLD_GUI_HIDE_ON_CLOSE	"gui_hide_on_close"
 #define FLD_GUI_AUTO_SHOW_INCOMING	"gui_auto_show_incoming"
 #define FLD_GUI_AUTO_SHOW_TIMEOUT	"gui_auto_show_timeout"
+#define FLD_GUI_BROWSER_CMD	"gui_browser_cmd"
 
 // Address book settings
 #define FLD_AB_SHOW_SIP_ONLY	"ab_show_sip_only"
@@ -79,11 +92,6 @@
 
 // Startup settings
 #define FLD_START_USER_PROFILE	"start_user_profile"
-#if 0
-// DEPRECATED
-#define FLD_START_USER_HOST	"start_user_host"
-#define FLD_START_USER_NIC	"start_user_nic"
-#endif
 #define FLD_START_HIDDEN	"start_hidden"
 
 // Network settings
@@ -112,6 +120,16 @@
 #define FLD_SHOW_BUDDY_LIST	"show_buddy_list"
 #define FLD_WARN_HIDE_USER	"warn_hide_user"
 
+// Settings to restore session after shutdown
+#define FLD_UI_SESSION_ID		"ui_session_id"
+#define FLD_UI_SESSION_ACTIVE_PROFILE	"ui_session_active_profile"
+#define FLD_UI_SESSION_MAIN_GEOMETRY	"ui_session_main_geometry"
+#define FLD_UI_SESSION_MAIN_HIDDEN	"ui_session_main_hidden"
+#define FLD_UI_SESSION_MAIN_STATE	"ui_session_main_state"
+
+// Mime settings
+#define FLD_MIME_SHARED_DATABASE	"mime_shared_database"
+
 /////////////////////////
 // class t_audio_device
 /////////////////////////
@@ -160,12 +178,50 @@ string t_audio_device::get_settings_value(void) const {
 	return s;
 }
 
+/////////////////////////
+// class t_win_geometry
+/////////////////////////
+
+t_win_geometry::t_win_geometry(int x_, int y_, int width_, int height_) :
+		x(x_), y(y_), width(width_), height(height_)
+{}
+
+t_win_geometry::t_win_geometry() :
+		x(0), y(0), width(0), height(0)
+{}
+
+t_win_geometry::t_win_geometry(const string &value) {
+	vector<string> v = split(value, ',');
+	
+	if (v.size() == 4) {
+		x = atoi(v[0].c_str());
+		y = atoi(v[1].c_str());
+		width = atoi(v[2].c_str());
+		height = atoi(v[3].c_str());
+	}
+}
+
+string t_win_geometry::encode(void) const {
+	string s;
+	
+	s = int2str(x);
+	s += ',';
+	s += int2str(y);
+	s += ',';
+	s += int2str(width);
+	s += ',';
+	s += int2str(height);
+	
+	return s;
+}
+
 
 /////////////////////////
 // class t_sys_settings
 /////////////////////////
 
 t_sys_settings::t_sys_settings() {
+	fd_lock_file = -1;
 	dir_share = DIR_SHARE;
 	filename = string(DIR_HOME);
 	filename += "/";
@@ -173,12 +229,17 @@ t_sys_settings::t_sys_settings() {
 	filename += "/";
 	filename += SYS_CONFIG_FILE;
 	
-	// OSS Default settings
+	// Audio device default settings
+#ifdef HAVE_LIBASOUND
+	dev_ringtone = audio_device(DEV_ALSA_DFLT);
+	dev_speaker = audio_device(DEV_ALSA_DFLT);
+	dev_mic = audio_device(DEV_ALSA_DFLT);
+#else
 	dev_ringtone = audio_device();
 	dev_speaker = audio_device();
 	dev_mic = audio_device();
+#endif
 	validate_audio_dev = true;
-	au_reduce_noise_mic = true;
 	alsa_play_period_size = 128;
 	alsa_capture_period_size = 32;
 	oss_fragment_size = 128;
@@ -205,11 +266,6 @@ t_sys_settings::t_sys_settings() {
 	hangup_both_3way = true;
 	
 	start_user_profiles.clear();
-#if 0
-	// DEPRECATED
-	start_user_host.clear();
-	start_user_nic.clear();
-#endif
 	start_hidden = false;
 	
 	config_sip_port = 5060;
@@ -236,6 +292,10 @@ t_sys_settings::t_sys_settings() {
 	compact_line_status = false;
 	show_buddy_list = true;
 	warn_hide_user = true;
+	
+	ui_session_id.clear();
+	
+	mime_shared_database = DFLT_SHARED_MIME_DB;
 }
 
 // Getters
@@ -271,14 +331,6 @@ bool t_sys_settings::get_validate_audio_dev(void) const {
 	return result;	
 }
 
-bool t_sys_settings::get_au_reduce_noise_mic(void) const {
-	bool result;
-	mtx_sys.lock();
-	result = au_reduce_noise_mic;
-	mtx_sys.unlock();
-	return result;	
-}
-
 int t_sys_settings::get_alsa_play_period_size(void) const {
 	int result;
 	mtx_sys.lock();
@@ -344,35 +396,28 @@ bool t_sys_settings::get_log_show_debug(void) const {
 }
 
 bool t_sys_settings::get_gui_use_systray(void) const {
-	bool result;
-	mtx_sys.lock();
-	result = gui_use_systray;
-	mtx_sys.unlock();
-	return result;	
+	t_mutex_guard guard(mtx_sys);
+	return gui_use_systray;
 }
 
 bool t_sys_settings::get_gui_auto_show_incoming(void) const {
-	bool result;
-	mtx_sys.lock();
-	result = gui_auto_show_incoming;
-	mtx_sys.unlock();
-	return result;
+	t_mutex_guard guard(mtx_sys);
+	return gui_auto_show_incoming;
 }
 
 int t_sys_settings::get_gui_auto_show_timeout(void) const {
-	int result;
-	mtx_sys.lock();
-	result = gui_auto_show_timeout;
-	mtx_sys.unlock();
-	return result;
+	t_mutex_guard guard(mtx_sys);
+	return gui_auto_show_timeout;
 }
 
 bool t_sys_settings::get_gui_hide_on_close(void) const {
-	bool result;
-	mtx_sys.lock();
-	result = gui_hide_on_close;
-	mtx_sys.unlock();
-	return result;	
+	t_mutex_guard guard(mtx_sys);
+	return gui_hide_on_close;
+}
+
+string t_sys_settings::get_gui_browser_cmd(void) const {
+	t_mutex_guard guard(mtx_sys);
+	return gui_browser_cmd;
 }
 
 bool t_sys_settings::get_ab_show_sip_only(void) const {
@@ -439,25 +484,6 @@ list<string> t_sys_settings::get_start_user_profiles(void) const {
 	return result;	
 }
 
-#if 0
-// DEPRECATED
-string t_sys_settings::get_start_user_host(void) const {
-	string result;
-	mtx_sys.lock();
-	result = start_user_host;
-	mtx_sys.unlock();
-	return result;	
-}
-
-string t_sys_settings::get_start_user_nic(void) const {
-	string result;
-	mtx_sys.lock();
-	result = start_user_nic;
-	mtx_sys.unlock();
-	return result;
-}
-#endif
-
 bool t_sys_settings::get_start_hidden(void) const {
 	bool result;
 	mtx_sys.lock();
@@ -608,14 +634,40 @@ bool t_sys_settings::get_show_buddy_list(void) const {
 	return result;
 }
 
-bool t_sys_settings::get_warn_hide_user(void) const {
-	bool result;
-	mtx_sys.lock();
-	result = warn_hide_user;
-	mtx_sys.unlock();
-	return result;
+string t_sys_settings::get_ui_session_id(void) const {
+	t_mutex_guard guard(mtx_sys);
+	return ui_session_id;
 }
 
+list<string> t_sys_settings::get_ui_session_active_profiles(void) const {
+	t_mutex_guard guard(mtx_sys);
+	return ui_session_active_profiles;
+}
+
+t_win_geometry t_sys_settings::get_ui_session_main_geometry(void) const {
+	t_mutex_guard guard(mtx_sys);
+	return ui_session_main_geometry;
+}
+
+bool t_sys_settings::get_ui_session_main_hidden(void) const {
+	t_mutex_guard guard(mtx_sys);
+	return ui_session_main_hidden;
+}
+
+unsigned int t_sys_settings::get_ui_session_main_state(void) const {
+	t_mutex_guard guard(mtx_sys);
+	return ui_session_main_state;
+}
+
+bool t_sys_settings::get_warn_hide_user(void) const {
+	t_mutex_guard guard(mtx_sys);
+	return warn_hide_user;
+}
+
+string t_sys_settings::get_mime_shared_database(void) const {
+	t_mutex_guard guard(mtx_sys);
+	return mime_shared_database;
+}
 
 // Setters
 void t_sys_settings::set_dev_ringtone(const t_audio_device &dev) {
@@ -642,12 +694,6 @@ void t_sys_settings::set_validate_audio_dev(bool b) {
 	mtx_sys.unlock();
 }
 
-void t_sys_settings::set_au_reduce_noise_mic(bool b) {
-	mtx_sys.lock();
-	au_reduce_noise_mic = b;
-	mtx_sys.unlock();
-}
-
 void t_sys_settings::set_alsa_play_period_size(int size) {
 	mtx_sys.lock();
 	alsa_play_period_size = size;
@@ -685,39 +731,38 @@ void t_sys_settings::set_log_show_stun(bool b) {
 }
 
 void t_sys_settings::set_log_show_memory(bool b) {
-	mtx_sys.lock();
+	t_mutex_guard guard(mtx_sys);
 	log_show_memory = b;
-	mtx_sys.unlock();
 }
 
 void t_sys_settings::set_log_show_debug(bool b) {
-	mtx_sys.lock();
+	t_mutex_guard guard(mtx_sys);
 	log_show_debug = b;
-	mtx_sys.unlock();
 }
 
 void t_sys_settings::set_gui_use_systray(bool b) {
-	mtx_sys.lock();
+	t_mutex_guard guard(mtx_sys);
 	gui_use_systray = b;
-	mtx_sys.unlock();
 }
 
 void t_sys_settings::set_gui_hide_on_close(bool b) {
-	mtx_sys.lock();
+	t_mutex_guard guard(mtx_sys);
 	gui_hide_on_close = b;
-	mtx_sys.unlock();
 }
 
 void t_sys_settings::set_gui_auto_show_incoming(bool b) {
-	mtx_sys.lock();
+	t_mutex_guard guard(mtx_sys);
 	gui_auto_show_incoming = b;
-	mtx_sys.unlock();
 }
 
 void t_sys_settings::set_gui_auto_show_timeout(int timeout) {
-	mtx_sys.lock();
+	t_mutex_guard guard(mtx_sys);
 	gui_auto_show_timeout = timeout;
-	mtx_sys.unlock();
+}
+
+void t_sys_settings::set_gui_browser_cmd(const string &s) {
+	t_mutex_guard guard(mtx_sys);
+	gui_browser_cmd = s;
 }
 
 void t_sys_settings::set_ab_show_sip_only(bool b) {
@@ -768,21 +813,6 @@ void t_sys_settings::set_start_user_profiles(const list<string> &profiles) {
 	mtx_sys.unlock();
 }
 
-#if 0
-// DEPRECATED
-void t_sys_settings::set_start_user_host(const string &host) {
-	mtx_sys.lock();
-	start_user_host = host;
-	mtx_sys.unlock();
-}
-
-void t_sys_settings::set_start_user_nic(const string &dev) {
-	mtx_sys.lock();
-	start_user_nic = dev;
-	mtx_sys.unlock();
-}
-#endif
-
 void t_sys_settings::set_start_hidden(bool b) {
 	mtx_sys.lock();
 	start_hidden = b;
@@ -907,12 +937,40 @@ void t_sys_settings::set_show_buddy_list(bool b) {
 	mtx_sys.unlock();
 }
 
-void t_sys_settings::set_warn_hide_user(bool b) {
-	mtx_sys.lock();
-	warn_hide_user = b;
-	mtx_sys.unlock();
+void t_sys_settings::set_ui_session_id(const string &id) {
+	t_mutex_guard guard(mtx_sys);
+	ui_session_id = id;
 }
 
+void t_sys_settings::set_ui_session_active_profiles(const list<string> &profiles) {
+	t_mutex_guard guard(mtx_sys);
+	ui_session_active_profiles = profiles;
+}
+
+void t_sys_settings::set_ui_session_main_geometry(const t_win_geometry &geometry) {
+	t_mutex_guard guard(mtx_sys);
+	ui_session_main_geometry = geometry;
+}
+
+void t_sys_settings::set_ui_session_main_hidden(bool hidden) {
+	t_mutex_guard guard(mtx_sys);
+	ui_session_main_hidden = hidden;
+}
+
+void t_sys_settings::set_ui_session_main_state(unsigned int state) {
+	t_mutex_guard guard(mtx_sys);
+	ui_session_main_state = state;
+}
+
+void t_sys_settings::set_warn_hide_user(bool b) {
+	t_mutex_guard guard(mtx_sys);
+	warn_hide_user = b;
+}
+
+void t_sys_settings::set_mime_shared_database(const string &filename) {
+	t_mutex_guard guard(mtx_sys);
+	mime_shared_database = filename;
+}
 
 string t_sys_settings::about(bool html) const {
 	string s = PRODUCT_NAME;
@@ -923,7 +981,7 @@ string t_sys_settings::about(bool html) const {
 	if (html) s += "<BR>";
 	s += "\n";
 	
-	s += "Copyright (C) 2005-2008  ";
+	s += "Copyright (C) 2005-2009  ";
 	s += PRODUCT_AUTHOR;
 	if (html) s += "<BR>";
 	s += "\n";
@@ -945,16 +1003,29 @@ string t_sys_settings::about(bool html) const {
 	if (html) s += "<BR>";
 	s += "\n";
 	
+	s += "* Werner Dittmann (ZRTP/SRTP)\n";
+	if (html) s += "<BR>";
+	
+	s += "* Bogdan Harjoc (AKAv1-MD5, Service-Route)\n";
+	if (html) s += "<BR>";
+	
+	s += "* Roman Imankulov (command line editing)\n";
+	if (html) s += "<BR>";
+	
 	if (html) {
-		s += "* ALSA - Rickard Petz&auml;ll";
-		s += "<BR>";
+		s += "* Ondrej Mori&scaron; (codec preprocessing)<BR>\n";
 	} else {
-		s += "* ALSA - Rickard Petzall";
+		s += "* Ondrej Moris (codec preprocessing)\n";
 	}
+	
+	if (html) {
+		s += "* Rickard Petz&auml;ll (ALSA)<BR>\n";
+	} else {
+		s += "* Rickard Petzall (ALSA)\n";
+	}
+	
+	if (html) s += "<BR>";
 	s += "\n";
-	s += "* ZRTP/SRTP - Werner Dittmann";
-	if (html) s += "<BR><BR>";
-	s += "\n\n";
 
 	s += TRANSLATE("This software contains the following software from 3rd parties:");		
 	if (html) s += "<BR>";
@@ -1126,9 +1197,7 @@ bool t_sys_settings::check_environment(string &error_msg) const {
 	}
 
 	// Check if user directory exists
-	dirname = DIR_HOME;
-	dirname += '/';
-	dirname += DIR_USER;
+	dirname = get_dir_user();
 	if (stat(dirname.c_str(), &stat_buf) != 0) {
 		// User directory does not exist. Create it now.
 		if (mkdir(dirname.c_str(), S_IRUSR | S_IWUSR | S_IXUSR) != 0) {
@@ -1139,6 +1208,19 @@ bool t_sys_settings::check_environment(string &error_msg) const {
 			return false;
 		}
 	}
+	
+	// Check if tmp file directory exists
+	dirname = get_dir_tmpfile();
+	if (stat(dirname.c_str(), &stat_buf) != 0) {
+		// Tmp file directory does not exist. Create it now.
+		if (mkdir(dirname.c_str(), S_IRUSR | S_IWUSR | S_IXUSR) != 0) {
+			// Failed to create the tmp file directory
+			error_msg = TRANSLATE("Cannot create directory %1 .");
+			error_msg = replace_first(error_msg, "%1", dirname);
+			mtx_sys.unlock();
+			return false;
+		}
+	}
 
 	mtx_sys.unlock();
 	return true;
@@ -1172,8 +1254,164 @@ string t_sys_settings::get_dir_user(void) const {
 	return dir;
 }
 
-bool t_sys_settings::create_lock_file(string &error_msg, bool &already_running) const {
-	struct stat stat_buf;
+string t_sys_settings::get_history_file(void) const {
+	string dir = get_dir_user();
+	dir += "/";
+	dir += FILE_CLI_HISTORY;
+	
+	return dir;
+}
+
+string t_sys_settings::get_dir_tmpfile(void) const {
+	string dir = get_dir_user();
+	dir += "/";
+	dir += DIR_TMPFILE;
+	
+	return dir;
+}
+
+bool t_sys_settings::is_tmpfile(const string &filename) const {
+	string tmpdir = get_dir_tmpfile();
+	
+	return filename.substr(0, tmpdir.size()) == tmpdir;
+}
+
+bool t_sys_settings::save_tmp_file(const string &data, const string &file_extension,
+		string &filename, string &error_msg) 
+{
+	string fname = get_dir_tmpfile();
+	fname += "/XXXXXX";
+	
+	char *tmpfile = strdup(fname.c_str());
+	MEMMAN_NEW(tmpfile);
+	int fd = mkstemp(tmpfile);
+	
+	if (fd < 0) {
+		error_msg = get_error_str(errno);
+		MEMMAN_DELETE(tmpfile);
+		free(tmpfile);
+		return false;
+	}
+	
+	close(fd);
+	ofstream f(tmpfile);
+	if (!f) {
+		error_msg = TRANSLATE("Failed to create file %1");
+		error_msg = replace_first(error_msg, "%1", tmpfile);
+		MEMMAN_DELETE(tmpfile);
+		free(tmpfile);
+		return false;
+	}
+	
+	f.write(data.c_str(), data.size());
+	if (!f.good()) {
+		error_msg = TRANSLATE("Failed to write data to file %1");
+		error_msg = replace_first(error_msg, "%1", tmpfile);
+		f.close();
+		MEMMAN_DELETE(tmpfile);
+		free(tmpfile);
+		return false;
+	}
+	
+	f.close();
+	
+	// Rename to name with extension
+	filename = apply_glob_to_filename(tmpfile, file_extension);
+	
+	if (rename(tmpfile, filename.c_str()) < 0) {
+		error_msg = get_error_str(errno);
+		MEMMAN_DELETE(tmpfile);
+		free(tmpfile);
+		return false;
+	}
+	
+	MEMMAN_DELETE(tmpfile);
+	free(tmpfile);
+	return true;
+}
+
+bool t_sys_settings::save_sip_body(const t_sip_message &sip_msg,
+		const string &suggested_file_extension,
+		string &tmpname, string &save_as_name, string &error_msg)
+{
+	bool retval = true;
+	
+	if (!sip_msg.body) {
+		error_msg = "Missing body";
+		return false;
+	}
+	
+	// Determine file extension and save-as name
+	// The algorithm to get the file extension (glob expression) is:
+	// 1) If the a file name is supplied in the Content-Disposition header, then
+	//    take the file extension from that file name.
+	// 2) If no extension is found, then take the suggested_file_extension
+	// 3) If still no file extension is found, then retrieve the file extension
+	//    from the t_media object in the Content-Type header. 
+	string file_ext = suggested_file_extension;
+	save_as_name.clear();
+	
+	if (sip_msg.hdr_content_disp.is_populated() &&
+	    sip_msg.hdr_content_disp.type == DISPOSITION_ATTACHMENT &&
+	    !sip_msg.hdr_content_disp.filename.empty()) 
+	{
+		string x = get_extension_from_filename(sip_msg.hdr_content_disp.filename);
+		if (!x.empty()) file_ext = string("*." + x);
+		
+		save_as_name = strip_path_from_filename(sip_msg.hdr_content_disp.filename);
+	}
+	if (file_ext.empty()) {
+		file_ext = sip_msg.hdr_content_type.media.get_file_glob();
+		
+		if (file_ext.empty()) {
+			file_ext = "*";
+		}
+	}
+	
+	// Avoid copy of opaque data
+	if (sip_msg.body->get_type() == BODY_OPAQUE) {
+		t_sip_body_opaque *body_opaque = dynamic_cast<t_sip_body_opaque *>(sip_msg.body);
+		retval = save_tmp_file(body_opaque->opaque, file_ext, tmpname, error_msg);
+	} else {
+		retval = save_tmp_file(sip_msg.body->encode(), file_ext, tmpname, error_msg);
+	}
+	
+	return retval;
+}
+
+void t_sys_settings::remove_all_tmp_files(void) const {
+	DIR *tmpdir = opendir(get_dir_tmpfile().c_str());
+	
+	if (!tmpdir) {
+		log_file->write_report(get_error_str(errno), "t_sys_settings::remove_all_tmp_files");
+		return;
+	}
+	
+	struct dirent *entry = readdir(tmpdir);
+	while (entry) {
+		if (strcmp(entry->d_name, ".") != 0 && strcmp(entry->d_name, "..") != 0) {
+			string fname = get_dir_tmpfile();
+			fname += PATH_SEPARATOR;
+			fname += entry->d_name;
+			
+			log_file->write_header("t_sys_settings::remove_all_tmp_files");
+			log_file->write_raw("Remove tmp file ");
+			log_file->write_raw(fname);
+			log_file->write_endl();
+			log_file->write_footer();
+			
+			unlink(fname.c_str());
+		}
+		
+		entry = readdir(tmpdir);
+	}
+	
+	closedir(tmpdir);
+}
+
+bool t_sys_settings::create_lock_file(bool shared_lock, string &error_msg, 
+                                      bool &already_running) 
+{
 	string lck_filename;
 	already_running = false;
 
@@ -1182,45 +1420,42 @@ bool t_sys_settings::create_lock_file(string &error_msg, bool &already_running)
         lck_filename += DIR_USER;
         lck_filename += "/";
         lck_filename += LOCK_FILENAME;
-
-	// Check if a lock file already exists
-	if (stat(lck_filename.c_str(), &stat_buf) == 0) {
-		ifstream f(lck_filename.c_str());
-		if (!f) {
-			error_msg =  TRANSLATE("Lock file %1 already exist, but cannot be opened.");
-			error_msg = replace_first(error_msg, "%1", lck_filename);
-			return false;
-		}
-
-		// Check if lock is stale
-		pid_t lock_pid;
-		f >> lock_pid;
-		if (kill(lock_pid, 0) == 0) {
-			// The pid in the lock file exists, so Twinkle is
-			// already running.
+	
+	fd_lock_file = open(lck_filename.c_str(), O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
+	if (fd_lock_file < 0) {
+		error_msg = TRANSLATE("Cannot create %1 .");
+		error_msg = replace_first(error_msg, "%1", lck_filename);
+		error_msg += "\n";
+		error_msg += get_error_str(errno);
+		return false;
+	}
+	
+	struct flock lock_options;
+	
+	// Try to acquire an exclusive lock
+	if (!shared_lock)
+	{
+		memset(&lock_options, 0, sizeof(struct flock));
+		lock_options.l_type = F_WRLCK;
+		lock_options.l_whence = SEEK_SET;
+		
+		if (fcntl(fd_lock_file, F_SETLK, &lock_options) < 0) {
 			already_running = true;
 			error_msg = TRANSLATE("%1 is already running.\nLock file %2 already exists.");
 			error_msg = replace_first(error_msg, "%1", PRODUCT_NAME);
 			error_msg = replace_first(error_msg, "%2", lck_filename);
 			return false;
 		}
-
-		// The lock is stale; delete it
-		f.close();
-		unlink(lck_filename.c_str());
 	}
-
-	// Create lock file
-	ofstream f(lck_filename.c_str());
-	if (!f) {
-		error_msg = TRANSLATE("Cannot create %1 .");
-		error_msg = replace_first(error_msg, "%1", lck_filename);
-		return false;
-	}
-
-	f << getpid();
-	if (!f.good()) {
-		error_msg = TRANSLATE("Cannot write to %1 .");
+	
+	// Convert the lock to a shared lock. If the user forces multiple
+	// instances of Twinkle to run, then each will have a shared lock.
+	memset(&lock_options, 0, sizeof(struct flock));
+	lock_options.l_type = F_RDLCK;
+	lock_options.l_whence = SEEK_SET;
+	
+	if (fcntl(fd_lock_file, F_SETLK, &lock_options) < 0) {
+		error_msg = TRANSLATE("Cannot lock %1 .");
 		error_msg = replace_first(error_msg, "%1", lck_filename);
 		return false;
 	}
@@ -1228,16 +1463,18 @@ bool t_sys_settings::create_lock_file(string &error_msg, bool &already_running)
 	return true;
 }
 
-void t_sys_settings::delete_lock_file(void) const {
-	string lck_filename;
-
-        lck_filename = DIR_HOME;
-        lck_filename += "/";
-        lck_filename += DIR_USER;
-        lck_filename += "/";
-        lck_filename += LOCK_FILENAME;
-
-	unlink(lck_filename.c_str());
+void t_sys_settings::delete_lock_file(void) {
+	if (fd_lock_file >= 0)
+	{
+		struct flock lock_options;
+		lock_options.l_type = F_UNLCK;
+		lock_options.l_whence = SEEK_SET;
+		
+		fcntl(fd_lock_file, F_SETLK, &lock_options);
+		
+		close(fd_lock_file);
+		fd_lock_file = -1;
+	}
 }
 
 bool t_sys_settings::read_config(string &error_msg) {
@@ -1302,8 +1539,6 @@ bool t_sys_settings::read_config(string &error_msg) {
 			dev_mic = audio_device(value);
 		} else if (parameter == FLD_VALIDATE_AUDIO_DEV) {
 			validate_audio_dev = yesno2bool(value);
-		} else if (parameter == FLD_AU_REDUCE_NOISE_MIC) {
-			au_reduce_noise_mic = yesno2bool(value);
 		} else if (parameter == FLD_ALSA_PLAY_PERIOD_SIZE) {
 			alsa_play_period_size = atoi(value.c_str());
 		} else if (parameter == FLD_ALSA_CAPTURE_PERIOD_SIZE) {
@@ -1328,6 +1563,8 @@ bool t_sys_settings::read_config(string &error_msg) {
 			gui_auto_show_incoming = yesno2bool(value);
 		} else if (parameter == FLD_GUI_AUTO_SHOW_TIMEOUT) {
 			gui_auto_show_timeout = atoi(value.c_str());
+		} else if (parameter == FLD_GUI_BROWSER_CMD) {
+			gui_browser_cmd = value;
 		} else if (parameter == FLD_AB_SHOW_SIP_ONLY) {
 			ab_show_sip_only = yesno2bool(value);
 		} else if (parameter == FLD_AB_LOOKUP_NAME) {
@@ -1344,13 +1581,6 @@ bool t_sys_settings::read_config(string &error_msg) {
 			hangup_both_3way = yesno2bool(value);
 		} else if (parameter == FLD_START_USER_PROFILE) {
 			if (!value.empty()) start_user_profiles.push_back(value);
-#if 0
-		// DEPRECATED
-		} else if (parameter == FLD_START_USER_HOST) {
-			start_user_host = value;
-		} else if (parameter == FLD_START_USER_NIC) {
-			start_user_nic = value;
-#endif
 		} else if (parameter == FLD_START_HIDDEN) {
 			start_hidden = yesno2bool(value);
 		} else if (parameter == FLD_sip_udp_port) { // Deprecated parameter
@@ -1394,8 +1624,20 @@ bool t_sys_settings::read_config(string &error_msg) {
 			//compact_line_status = yesno2bool(value);
 		} else if (parameter == FLD_SHOW_BUDDY_LIST) {
 			show_buddy_list = yesno2bool(value);
+		} else if (parameter == FLD_UI_SESSION_ID) {
+			ui_session_id = value;
+		} else if (parameter == FLD_UI_SESSION_ACTIVE_PROFILE) {
+			ui_session_active_profiles.push_back(value);
+		} else if (parameter == FLD_UI_SESSION_MAIN_GEOMETRY) {
+			ui_session_main_geometry = value;
+		} else if (parameter == FLD_UI_SESSION_MAIN_HIDDEN) {
+			ui_session_main_hidden = yesno2bool(value);
+		} else if (parameter == FLD_UI_SESSION_MAIN_STATE) {
+			ui_session_main_state = atoi(value.c_str());
 		} else if (parameter == FLD_WARN_HIDE_USER) {
 			warn_hide_user = yesno2bool(value);
+		} else if (parameter == FLD_MIME_SHARED_DATABASE) {
+			mime_shared_database = value;
 		}
 			
 		// Unknown field names are skipped.
@@ -1441,7 +1683,6 @@ bool t_sys_settings::write_config(string &error_msg) {
 	config << FLD_DEV_SPEAKER << '=' << dev_speaker.get_settings_value() << endl;
 	config << FLD_DEV_MIC << '=' << dev_mic.get_settings_value() << endl;
 	config << FLD_VALIDATE_AUDIO_DEV << '=' << bool2yesno(validate_audio_dev) << endl;
-	config << FLD_AU_REDUCE_NOISE_MIC << '=' << bool2yesno(au_reduce_noise_mic) << endl;
 	config << FLD_ALSA_PLAY_PERIOD_SIZE << '=' << alsa_play_period_size << endl;
 	config << FLD_ALSA_CAPTURE_PERIOD_SIZE << '=' << alsa_capture_period_size << endl;
 	config << FLD_OSS_FRAGMENT_SIZE << '=' << oss_fragment_size << endl;
@@ -1462,6 +1703,7 @@ bool t_sys_settings::write_config(string &error_msg) {
 	config << FLD_GUI_HIDE_ON_CLOSE << '=' << bool2yesno(gui_hide_on_close) << endl;
 	config << FLD_GUI_AUTO_SHOW_INCOMING << '=' << bool2yesno(gui_auto_show_incoming) << endl;
 	config << FLD_GUI_AUTO_SHOW_TIMEOUT << '=' << gui_auto_show_timeout << endl;
+	config << FLD_GUI_BROWSER_CMD << '=' << gui_browser_cmd << endl;
 	config << endl;
 	
 	// Write address book settings
@@ -1491,11 +1733,7 @@ bool t_sys_settings::write_config(string &error_msg) {
 	{
 		config << FLD_START_USER_PROFILE << '=' << *i << endl;
 	}
-#if 0
-	// DEPRECATED
-	config << FLD_START_USER_HOST << '=' << start_user_host << endl;
-	config << FLD_START_USER_NIC  << '=' << start_user_nic << endl;
-#endif
+
 	config << FLD_START_HIDDEN << '=' << bool2yesno(start_hidden) << endl;
 	config << endl;
 	
@@ -1515,6 +1753,11 @@ bool t_sys_settings::write_config(string &error_msg) {
 	config << FLD_RINGBACK_FILE << '=' << ringback_file << endl;
 	config << endl;
 	
+	// Write MIME settings
+	config << "# MIME settings\n";
+	config << FLD_MIME_SHARED_DATABASE << '=' << mime_shared_database << endl;
+	config << endl;
+	
 	// Write persistent user interface state
 	config << "# Persistent user interface state\n";
 	config << FLD_LAST_USED_PROFILE << '=' << last_used_profile << endl;
@@ -1536,6 +1779,22 @@ bool t_sys_settings::write_config(string &error_msg) {
 	
 	config << endl;
 	
+	// Write session settins
+	config << "# UI session settings\n";
+	config << FLD_UI_SESSION_ID << '=' << ui_session_id << endl;
+
+	for (list<string>::iterator i = ui_session_active_profiles.begin();
+	     i != ui_session_active_profiles.end(); i++)
+	{
+		config << FLD_UI_SESSION_ACTIVE_PROFILE << '=' << *i << endl;
+	}
+	
+	config << FLD_UI_SESSION_MAIN_GEOMETRY << '=' << ui_session_main_geometry.encode() << endl;
+	config << FLD_UI_SESSION_MAIN_HIDDEN << '=' << bool2yesno(ui_session_main_hidden) << endl;
+	config << FLD_UI_SESSION_MAIN_STATE << '=' << ui_session_main_state << endl;
+	
+	config << endl;
+	
 	// Check if writing succeeded
 	if (!config.good()) {
 		// Restore backup
