@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2008  Michel de Boer <michel@twinklephone.com>
+    Copyright (C) 2005-2009  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,34 +22,46 @@
 #include <cstdlib>
 #include <string>
 #include <list>
+#include "parser/sip_message.h"
 #include "sockets/url.h"
 #include "threads/mutex.h"
 #include "twinkle_config.h"
 
 using namespace std;
 
-// General system settings
-
-// User directory, relative to the home directory ($HOME)
+/** @name General system settings */
+//@{
+/** User directory, relative to the home directory ($HOME) */
 #define DIR_USER	".twinkle"
 
-// Home directory
+/** Home directory */
 #define DIR_HOME	(getenv("HOME"))
 
-// Device file for DSP
+/** Directory for storing temporary files, relative to @ref DIR_USER */
+#define DIR_TMPFILE	"tmp"
+
+/** Device file for DSP */
 #define DEV_DSP		"/dev/dsp"
 
-// Device prefixes in settings file
+/** Device prefixes in settings file */
 #define PFX_OSS		"oss:"
 #define PFX_ALSA	"alsa:"
 
-// Device string for other device
+/** ALSA default device */
+#define DEV_ALSA_DFLT	"alsa:default"
+
+/** Device string for other device */
 #define DEV_OTHER	"other device"
 
-// File with SIP providers for the wizard
+/** File with SIP providers for the wizard */
 #define FILE_PROVIDERS	"providers.csv"
 
+/** File with CLI command history */
+#define FILE_CLI_HISTORY "twinkle.history"
+//@}
 
+
+/** Audio device */
 class t_audio_device {
 public:
 	enum t_audio_device_type {
@@ -66,11 +78,42 @@ public:
 	string get_settings_value(void) const;
 };
 
+/** Window geometry */
+struct t_win_geometry {
+	int x;		/**< x-coordinate of top left corner */
+	int y;		/**< y-coordinate of top left corner */
+	int width;	/**< Window width */
+	int height;	/**< Window height */
+	
+	/** Constructor */
+	t_win_geometry();
+	
+	/** Constructor */
+	t_win_geometry(int x_, int y_, int width_, int height_);
+	
+	/**
+	 * Construct a geometry from an encoded string.
+	 * If the string cannot be parsed, all values are set to zero.
+	 * @param value [in] Encoded string "x,y,widht,height"
+	 */
+	t_win_geometry(const string &value);
+	
+	/**
+	 * Encode geometry into a string.
+	 * @return Encoded geometry "x,y,width,height"
+	 */
+	string encode(void) const;
+};
+
+/** System settings */
 class t_sys_settings {
 private:
 	// Mutex to avoid sync concurrent access
 	mutable t_recursive_mutex	mtx_sys;
 	
+	/** File descriptor of lock file */
+	int fd_lock_file;
+	
 	// Share directory for files applicable to all users
 	string		dir_share;
 	
@@ -89,7 +132,6 @@ private:
 	// usage.
 	bool			validate_audio_dev;
 	
-	bool			au_reduce_noise_mic;
 	int			alsa_play_period_size;
 	int			alsa_capture_period_size;
 	int			oss_fragment_size;
@@ -101,14 +143,19 @@ private:
 	bool		log_show_memory;
 	bool		log_show_debug;
 	
-	// GUI settings
+	/** @name GUI settings */
+	//@{
 	bool		gui_use_systray;
 	bool		gui_hide_on_close;
 	
-	// Show main window on incoming call after a few seconds
+	/** Show main window on incoming call after a few seconds */
 	bool		gui_auto_show_incoming;
 	int		gui_auto_show_timeout;
 	
+	/** Command to start an internet browser */
+	string		gui_browser_cmd;
+	//@}
+	
 	// Address book settings
 	bool		ab_show_sip_only;
 	bool		ab_lookup_name;
@@ -129,15 +176,12 @@ private:
 	
 	// Startup settings
 	list<string>	start_user_profiles;
-	
-#if 0
-	// DEPRECATED
-	string		start_user_host;
-	string		start_user_nic;
-#endif
 
 	bool		start_hidden;
 	
+	/** The full path name of the shared mime database */
+	string		mime_shared_database;
+	
 	/** @name Network settings */
 	//@{
 	/** Port for sending and receiving SIP messages. This is the value
@@ -206,18 +250,37 @@ private:
 	bool		show_buddy_list;
 	//@}
 	
+	/** @name Settings to restore a previous user interface session after system shutdown */
+	//@{
+	/** ID of previous session */
+	string		ui_session_id;
+	
+	/** Active user profiles */
+	list<string>	ui_session_active_profiles;
+	
+	/** Geometry of main window */
+	t_win_geometry	ui_session_main_geometry;
+	
+	/** Flag to indicate if the main window is hidden. */
+	bool		ui_session_main_hidden;
+	
+	/** Window state of main window. */
+	unsigned int	ui_session_main_state;
+	//@}
+	
 	// One time warnings
 	bool		warn_hide_user; // Warn use that provider may not support hiding.
 	
 public:
+	/** Constructor */
 	t_sys_settings();
 	
-	// Getters
+	/** @name Getters */
+	//@{
 	t_audio_device get_dev_ringtone(void) const;
 	t_audio_device get_dev_speaker(void) const;
 	t_audio_device get_dev_mic(void) const;
 	bool get_validate_audio_dev(void) const;
-	bool get_au_reduce_noise_mic(void) const;
 	int get_alsa_play_period_size(void) const;
 	int get_alsa_capture_period_size(void) const;
 	int get_oss_fragment_size(void) const;
@@ -230,6 +293,7 @@ public:
 	bool get_gui_hide_on_close(void) const;
 	bool get_gui_auto_show_incoming(void) const;
 	int get_gui_auto_show_timeout(void) const;
+	string get_gui_browser_cmd(void) const;
 	bool get_ab_show_sip_only(void) const;
 	bool get_ab_lookup_name(void) const;
 	bool get_ab_override_display(void) const;
@@ -238,11 +302,6 @@ public:
 	bool get_call_waiting(void) const;
 	bool get_hangup_both_3way(void) const;
 	list<string> get_start_user_profiles(void) const;
-#if 0
-	// DEPRECATED
-	string get_start_user_host(void) const;
-	string get_start_user_nic(void) const;
-#endif
 	bool get_start_hidden(void) const;
 	unsigned short get_config_sip_port(void) const;
 	unsigned short get_rtp_port(void) const;
@@ -262,14 +321,21 @@ public:
 	bool get_show_display(void) const;
 	bool get_compact_line_status(void) const;
 	bool get_show_buddy_list(void) const;
+	string get_ui_session_id(void) const;
+	list<string> get_ui_session_active_profiles(void) const;
+	t_win_geometry get_ui_session_main_geometry(void) const;
+	bool get_ui_session_main_hidden(void) const;
+	unsigned int get_ui_session_main_state(void) const;
 	bool get_warn_hide_user(void) const;
+	string get_mime_shared_database(void) const;
+	//@}
 	
-	// Setters
+	/** @name Setters */
+	//@{
 	void set_dev_ringtone(const t_audio_device &dev);
 	void set_dev_speaker(const t_audio_device &dev);
 	void set_dev_mic(const t_audio_device &dev);
 	void set_validate_audio_dev(bool b);
-	void set_au_reduce_noise_mic(bool b);
 	void set_alsa_play_period_size(int size);
 	void set_alsa_capture_period_size(int size);
 	void set_oss_fragment_size(int size);
@@ -282,6 +348,7 @@ public:
 	void set_gui_hide_on_close(bool b);
 	void set_gui_auto_show_incoming(bool b);
 	void set_gui_auto_show_timeout(int timeout);
+	void set_gui_browser_cmd(const string &s);
 	void set_ab_show_sip_only(bool b);
 	void set_ab_lookup_name(bool b);
 	void set_ab_override_display(bool b);
@@ -290,11 +357,6 @@ public:
 	void set_call_waiting(bool b);
 	void set_hangup_both_3way(bool b);
 	void set_start_user_profiles(const list<string> &profiles);
-#if 0
-	// DEPRECATED
-	void set_start_user_host(const string &host);
-	void set_start_user_nic(const string &dev);
-#endif
 	void set_start_hidden(bool b);
 	void set_config_sip_port(unsigned short port);
 	void set_override_sip_port(unsigned short port);
@@ -316,37 +378,131 @@ public:
 	void set_show_display(bool b);
 	void set_compact_line_status(bool b);
 	void set_show_buddy_list(bool b);
+	void set_ui_session_id(const string &id);
+	void set_ui_session_active_profiles(const list<string> &profiles);
+	void set_ui_session_main_geometry(const t_win_geometry &geometry);
+	void set_ui_session_main_hidden(bool hidden);
+	void set_ui_session_main_state(unsigned int state);
 	void set_warn_hide_user(bool b);
+	void set_mime_shared_database(const string &filename);
+	//@}
 	
-	// Return "about" text
+	/** 
+	 * Get "about" text.
+	 * @param html [in] Indicates if "about" text must be in HTML format.
+	 * @return The "about" text"
+	 */
 	string about(bool html) const;
 	
-	// Return product release date in locale format
+	/**
+	 * Get produce release date.
+	 * @return product release date in locale format
+	 */
 	string get_product_date(void) const;
 	
-	// Return a string of options that are built, e.g. ALSA, KDE
+	/** 
+	 * Get a string of options that are built, e.g. ALSA, KDE
+	 * @return The string of options.
+	 */
 	string get_options_built(void) const;
 
-	// Check if the environment of the machine satisfies all requirements.
-	// If not, then false is returned and error_msg contains an appropriate
-	// error message to show the user.
+	/** 
+	 * Check if the environment of the machine satisfies all requirements.
+	 * @param error_msg [out] User readable error message when false is returned.
+	 * @return true if all requirements are met.
+	 * @return false, otherwise and error_msg contains an appropriate
+	 * error message to show the user.
+	 */
 	bool check_environment(string &error_msg) const;
 
-	// Set the share directory
+	/**
+	 * Set the share directory
+	 * @param dir [in] Absolute path of the share directory.
+	 */ 
 	void set_dir_share(const string &dir);
 
-	// Get the share directory
+	/**
+	 * Get the share directory.
+	 * @return Absolute path of the directory with shared files.
+	 */
 	string get_dir_share(void) const;
 	
-	// Get the directory containing language translation files
+	/**
+	 * Get the directory containing language translation files.
+	 * @return Absolute path of the language directory.
+	 */
 	string get_dir_lang(void) const;
 	
-	// Get the user directory
+	/**
+	 * Get the user directory.
+	 * @return Absolute path of the user directory.
+	 */
 	string get_dir_user(void) const;
+	
+	/**
+	 * Get the CLI command history file.
+	 * @return Full pathname of the history file.
+	 */
+	string get_history_file(void) const;
+	
+	/** 
+	 * Get the temporary file directory.
+	 * @return The full pathname of the temporary file directory.
+	 */
+	string get_dir_tmpfile(void) const;
+	
+	/**
+	 * Check if a file is located in the temporary file directory.
+	 * @return true if the file is in the temporary file directory, false otherwise.
+	 */
+	bool is_tmpfile(const string &filename) const;
+	
+	/**
+	 * Save data to a temporary file.
+	 * @param data [in] Data to save.
+	 * @param file_extension [in] Extension (glob) for file name.
+	 * @param filename [out] File name of save file, relative to the tmp directory.
+	 * @param error_msg [out] If saving failed, then this parameter contains an
+	 *        error message.
+	 * @return true if saving succeeded, false otherwise.
+	 */
+	bool save_tmp_file(const string &data, const string &file_extension,
+		string &filename, string &error_msg);
+		
+	/**
+	 * Save the body of a SIP message to a temporary file.
+	 * @param sip_msg [in] The SIP message from which the body must be saved.
+	 * @param suggested_file_extension [in] File extension (glob) for file name to save
+	 *        if an extension cannot be determined from a filename supplied as
+	 *        in the Content-Disposition header.
+	 * @param tmpname [out] The name of the saved file.
+	 * @param save_as_name [out] Suggested file name for user for saving.
+	 * @param error_msg [out] Error message when saving failed.
+	 * @return true if saving succeeded, false otherwise.
+	 */
+	bool save_sip_body(const t_sip_message &sip_msg,
+		const string &suggested_file_extension,
+		string &tmpname, string &save_as_name, string &error_msg);
+		
+	/** Remove all files from the temporary file directory */
+	void remove_all_tmp_files(void) const;
 
-	// Lock file operations
-	bool create_lock_file(string &error_msg, bool &already_running) const;
-	void delete_lock_file(void) const;
+	/** @name Lock file operations */
+	/**
+	 * Create a lock file if it does not exist yet and take a file lock on it.
+	 * @param shared_lock [in] Indicates if the file lock must be shared or exclusive.
+	 *        A shared lock is needed when the users forces multiple Twinkle processes
+	 *        to run.
+	 * @param error_msg [out] Error message if the operation fails.
+	 * @param already_running [out] If the operation fails, this flag indicates Twinkle
+	 *        is already running.
+	 * @return True if the file is locked succesfully.
+	 * @return False if the file could not be locked.
+	 */
+	bool create_lock_file(bool shared_lock, string &error_msg, bool &already_running);
+	
+	/** Unlock the lock file. */
+	void delete_lock_file(void);
 	
 	// Read and parse a config file into the t_sys_settings object.
 	// Returns false if it fails. error_msg is an error message that can
