@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2008  Michel de Boer <michel@twinklephone.com>
+    Copyright (C) 2005-2009  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -70,20 +70,6 @@ void t_transaction_layer::recvd_request(t_request *r, t_tid tid,
 	t_response *resp;
 
 	lock();
-	
-	// If a message exceeded the maximum message size, than the body
-	// is not parsed by the listener.
-	if (r->hdr_content_length.is_populated() &&
-	    r->hdr_content_length.length > 0 &&
-	    !r->body)
-	{
-		resp = r->create_response(R_513_MESSAGE_TOO_LARGE);
-		send_response(resp, 0, tid);
-		MEMMAN_DELETE(resp);
-		delete resp;
-		unlock();
-		return;
-	}
 
 	// Return a 400 response if the SIP headers are wrong
 	if (!r->is_valid(fatal, reason)) {
@@ -105,6 +91,20 @@ void t_transaction_layer::recvd_request(t_request *r, t_tid tid,
 		return;
 	}
 	
+	// If a message exceeded the maximum message size, than the body
+	// is not parsed by the listener.
+	if (r->hdr_content_length.is_populated() &&
+	    r->hdr_content_length.length > 0 &&
+	    !r->body)
+	{
+		resp = r->create_response(R_513_MESSAGE_TOO_LARGE);
+		send_response(resp, 0, tid);
+		MEMMAN_DELETE(resp);
+		delete resp;
+		unlock();
+		return;
+	}
+	
 	// RFC 3261 8.2.3
 	// Return a 415 response if content encoding is not supported
 	if (r->body && r->hdr_content_encoding.is_populated()) {
@@ -221,6 +221,7 @@ void t_transaction_layer::run(void) {
 	t_event_failure		*ev_failure;
 	t_event_stun_response	*ev_stun_resp;
 	t_event_async_response	*ev_async_resp;
+	t_event_broken_connection *ev_broken_connection;
 	t_sip_message		*msg;
 	StunMessage		*stun_msg;
 	t_tid			tid;
@@ -275,6 +276,10 @@ void t_transaction_layer::run(void) {
 			ev_async_resp = dynamic_cast<t_event_async_response *>(event);
 			recvd_async_response(ev_async_resp);
 			break;
+		case EV_BROKEN_CONNECTION:
+			ev_broken_connection = dynamic_cast<t_event_broken_connection *>(event);
+			handle_broken_connection(ev_broken_connection);
+			break;
 		case EV_QUIT:
 			quit = true;
 			break;
