@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2008  Michel de Boer <michel@twinklephone.com>
+    Copyright (C) 2005-2009  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,12 +18,14 @@
 
 #include <cassert>
 #include <cstdlib>
+#include <cstring>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fstream>
 #include <iostream>
 #include <list>
+#include "diamondcard.h"
 #include "log.h"
 #include "phone.h"
 #include "twinkle_config.h"
@@ -48,6 +50,8 @@ extern t_phone		*phone;
 #define FLD_AUTH_REALM			"auth_realm"
 #define FLD_AUTH_NAME			"auth_name"
 #define FLD_AUTH_PASS			"auth_pass"
+#define FLD_AUTH_AKA_OP			"auth_aka_op"
+#define FLD_AUTH_AKA_AMF		"auth_aka_amf"
 
 // SIP SERVER fields
 #define FLD_OUTBOUND_PROXY		"outbound_proxy"
@@ -70,10 +74,15 @@ extern t_phone		*phone;
 #define FLD_SPEEX_BIT_RATE_TYPE		"speex_bit_rate_type"
 #define FLD_SPEEX_ABR_NB		"speex_abr_nb"
 #define FLD_SPEEX_ABR_WB		"speex_abr_wb"
-#define FLD_SPEEX_VAD			"speex_vad"
 #define FLD_SPEEX_DTX			"speex_dtx"
 #define FLD_SPEEX_PENH			"speex_penh"
+#define FLD_SPEEX_QUALITY		"speex_quality"
 #define FLD_SPEEX_COMPLEXITY		"speex_complexity"
+#define FLD_SPEEX_DSP_VAD		"speex_dsp_vad"
+#define FLD_SPEEX_DSP_AGC		"speex_dsp_agc"
+#define FLD_SPEEX_DSP_AGC_LEVEL		"speex_dsp_agc_level"
+#define FLD_SPEEX_DSP_AEC		"speex_dsp_aec"
+#define FLD_SPEEX_DSP_NRD		"speex_dsp_nrd"
 #define FLD_ILBC_PAYLOAD_TYPE		"ilbc_payload_type"
 #define FLD_ILBC_MODE			"ilbc_mode"
 #define FLD_G726_16_PAYLOAD_TYPE	"g726_16_payload_type"
@@ -107,6 +116,7 @@ extern t_phone		*phone;
 #define FLD_ASK_USER_TO_REFER		"ask_user_to_refer"
 #define FLD_AUTO_REFRESH_REFER_SUB	"auto_refresh_refer_sub"
 #define FLD_ATTENDED_REFER_TO_AOR	"attended_refer_to_aor"
+#define FLD_ALLOW_XFER_CONSULT_INPROG	"allow_xfer_consult_inprog"
 #define FLD_SEND_P_PREFERRED_ID		"send_p_preferred_id"
 
 // Transport/NAT fields
@@ -114,16 +124,20 @@ extern t_phone		*phone;
 #define FLD_SIP_TRANSPORT_UDP_THRESHOLD	"sip_transport_udp_threshold"
 #define FLD_NAT_PUBLIC_IP		"nat_public_ip"
 #define FLD_STUN_SERVER			"stun_server"
+#define FLD_PERSISTENT_TCP		"persistent_tcp"
+#define FLD_ENABLE_NAT_KEEPALIVE	"enable_nat_keepalive"
 
 // TIMER fields
 #define FLD_TIMER_NOANSWER		"timer_noanswer"
 #define FLD_TIMER_NAT_KEEPALIVE		"timer_nat_keepalive"
+#define FLD_TIMER_TCP_PING		"timer_tcp_ping"
 
 // ADDRESS FORMAT fields
 #define FLD_DISPLAY_USERONLY_PHONE	"display_useronly_phone"
 #define FLD_NUMERICAL_USER_IS_PHONE	"numerical_user_is_phone"
 #define FLD_REMOVE_SPECIAL_PHONE_SYM	"remove_special_phone_symbols"
 #define FLD_SPECIAL_PHONE_SYMBOLS	"special_phone_symbols"
+#define FLD_USE_TEL_URI_FOR_PHONE	"use_tel_uri_for_phone"
 
 // Ring tone settings
 #define FLD_USER_RINGTONE_FILE		"ringtone_file"
@@ -158,6 +172,7 @@ extern t_phone		*phone;
 
 // INSTANT MESSAGE
 #define FLD_IM_MAX_SESSIONS		"im_max_sessions"
+#define FLD_IM_SEND_ISCOMPOSING		"im_send_iscomposing"
 
 // PRESENCE
 #define FLD_PRES_SUBSCRIPTION_TIME	"pres_subscription_time"
@@ -336,6 +351,8 @@ bool t_user::set_server_value(t_url &server, const string &scheme, const string
 
 t_user::t_user() {
 	// Set defaults
+	memset(auth_aka_op, 0, AKA_OPLEN);
+	memset(auth_aka_amf, 0, AKA_AMFLEN);
 	use_outbound_proxy = false;
 	all_requests_to_proxy = false;
 	non_resolvable_to_proxy = false;
@@ -357,6 +374,8 @@ t_user::t_user() {
 	hold_variant = HOLD_RFC3264;
 	use_nat_public_ip = false;
 	use_stun = false;
+	persistent_tcp = true;
+	enable_nat_keepalive = false;
 	register_at_startup = true;
 	reg_add_qvalue = false;
 	reg_qvalue = 1.0;
@@ -372,6 +391,7 @@ t_user::t_user() {
 	max_redirections = 5;
 	timer_noanswer = 30;
 	timer_nat_keepalive = DUR_NAT_KEEPALIVE;
+	timer_tcp_ping = DUR_TCP_PING;
 	ext_100rel = EXT_SUPPORTED;
 	ext_replaces = true;
 	speex_nb_payload_type = 97;
@@ -380,10 +400,15 @@ t_user::t_user() {
 	speex_bit_rate_type = BIT_RATE_CBR;
 	speex_abr_nb = 0;
 	speex_abr_wb = 0;
-	speex_vad = true;
 	speex_dtx = false;
 	speex_penh = true;
-	speex_complexity = 2;
+	speex_quality = 6;
+	speex_complexity = 3;
+	speex_dsp_vad = true;
+	speex_dsp_agc = true;
+	speex_dsp_aec = false;
+	speex_dsp_nrd = true;
+	speex_dsp_agc_level = 20;
 	ilbc_payload_type = 96;
 	ilbc_mode = 30;
 	g726_16_payload_type = 102;
@@ -400,12 +425,14 @@ t_user::t_user() {
 	numerical_user_is_phone = false;
 	remove_special_phone_symbols = true;
 	special_phone_symbols = SPECIAL_PHONE_SYMBOLS;
+	use_tel_uri_for_phone = false;
 	referee_hold = false;
 	referrer_hold = true;
 	allow_refer = true;
 	ask_user_to_refer = true;
 	auto_refresh_refer_sub = false;
 	attended_refer_to_aor = false;
+	allow_transfer_consultation_inprog = false;
 	send_p_preferred_id = false;
 	sip_transport = SIP_TRANS_AUTO;
 	sip_transport_udp_threshold = 1300; // RFC 3261 18.1.1
@@ -430,6 +457,7 @@ t_user::t_user() {
 	mwi_subscription_time = 3600;
 	mwi_vm_address.clear();
 	im_max_sessions = 10;
+	im_send_iscomposing = true;
 	pres_subscription_time = 3600;
 	pres_publication_time = 3600;
 	pres_publish_startup = true;
@@ -446,6 +474,8 @@ t_user::t_user(const t_user &u) {
 	auth_realm = u.auth_realm;
 	auth_name = u.auth_name;
 	auth_pass = u.auth_pass;
+	memcpy(auth_aka_op, u.auth_aka_op, AKA_OPLEN);
+	memcpy(auth_aka_amf, u.auth_aka_amf, AKA_AMFLEN);
 	use_outbound_proxy = u.use_outbound_proxy;
 	outbound_proxy = u.outbound_proxy;
 	all_requests_to_proxy = u.all_requests_to_proxy;
@@ -466,10 +496,15 @@ t_user::t_user(const t_user &u) {
 	speex_bit_rate_type = u.speex_bit_rate_type;
 	speex_abr_nb = u.speex_abr_nb;
 	speex_abr_wb = u.speex_abr_wb;
-	speex_vad = u.speex_vad;
 	speex_dtx = u.speex_dtx;
 	speex_penh = u.speex_penh;
+	speex_quality = u.speex_quality;
 	speex_complexity = u.speex_complexity;
+	speex_dsp_vad = u.speex_dsp_vad;
+	speex_dsp_agc = u.speex_dsp_agc;
+	speex_dsp_agc_level = u.speex_dsp_agc_level;
+	speex_dsp_aec = u.speex_dsp_aec;
+	speex_dsp_nrd = u.speex_dsp_nrd;
 	ilbc_payload_type = u.ilbc_payload_type;
 	ilbc_mode = u.ilbc_mode;
 	g726_16_payload_type = u.g726_16_payload_type;
@@ -501,6 +536,7 @@ t_user::t_user(const t_user &u) {
 	ask_user_to_refer = u.ask_user_to_refer;
 	auto_refresh_refer_sub = u.auto_refresh_refer_sub;
 	attended_refer_to_aor = u.attended_refer_to_aor;
+	allow_transfer_consultation_inprog = u.allow_transfer_consultation_inprog;
 	send_p_preferred_id = u.send_p_preferred_id;
 	sip_transport = u.sip_transport;
 	sip_transport_udp_threshold = u.sip_transport_udp_threshold;
@@ -508,12 +544,16 @@ t_user::t_user(const t_user &u) {
 	nat_public_ip = u.nat_public_ip;
 	use_stun = u.use_stun;
 	stun_server = u.stun_server;
+	persistent_tcp = u.persistent_tcp;
+	enable_nat_keepalive = u.enable_nat_keepalive;
 	timer_noanswer = u.timer_noanswer;
 	timer_nat_keepalive = u.timer_nat_keepalive; 
+	timer_tcp_ping = u.timer_tcp_ping;
 	display_useronly_phone = u.display_useronly_phone;
 	numerical_user_is_phone = u.numerical_user_is_phone;
 	remove_special_phone_symbols = u.remove_special_phone_symbols;
 	special_phone_symbols = u.special_phone_symbols;
+	use_tel_uri_for_phone = u.use_tel_uri_for_phone;
 	ringtone_file = u.ringtone_file;
 	ringback_file = u.ringback_file;
 	script_incoming_call = u.script_incoming_call;
@@ -536,6 +576,7 @@ t_user::t_user(const t_user &u) {
 	mwi_subscription_time = u.mwi_subscription_time;
 	mwi_vm_address = u.mwi_vm_address;
 	im_max_sessions = u.im_max_sessions;
+	im_send_iscomposing = u.im_send_iscomposing;
 	pres_subscription_time = u.pres_subscription_time;
 	pres_publication_time = u.pres_publication_time;
 	pres_publish_startup = u.pres_publish_startup;
@@ -607,6 +648,16 @@ string t_user::get_auth_pass(void) const {
 	return result;
 }
 
+void t_user::get_auth_aka_op(uint8 *aka_op) const {
+	t_mutex_guard guard(mtx_user);
+	memcpy(aka_op, auth_aka_op, AKA_OPLEN);
+}
+	
+void t_user::get_auth_aka_amf(uint8 *aka_amf) const {
+	t_mutex_guard guard(mtx_user);
+	memcpy(aka_amf, auth_aka_amf, AKA_AMFLEN);
+}
+
 bool t_user::get_use_outbound_proxy(void) const {
 	bool result;
 	mtx_user.lock();
@@ -767,14 +818,6 @@ int t_user::get_speex_abr_wb(void) const {
 	return result;
 }
 
-bool t_user::get_speex_vad(void) const {
-	bool result;
-	mtx_user.lock();
-	result = speex_vad;
-	mtx_user.unlock();
-	return result;
-}
-
 bool t_user::get_speex_dtx(void) const {
 	bool result;
 	mtx_user.lock();
@@ -791,6 +834,14 @@ bool t_user::get_speex_penh(void) const {
 	return result;
 }
 
+unsigned short t_user::get_speex_quality(void) const {
+	unsigned short result;
+	mtx_user.lock();
+	result = speex_quality;
+	mtx_user.unlock();
+	return result;
+}
+
 unsigned short t_user::get_speex_complexity(void) const {
 	unsigned short result;
 	mtx_user.lock();
@@ -799,6 +850,46 @@ unsigned short t_user::get_speex_complexity(void) const {
 	return result;
 }
 
+bool t_user::get_speex_dsp_vad(void) const {
+	bool result;
+	mtx_user.lock();
+	result = speex_dsp_vad;
+	mtx_user.unlock();
+	return result;
+}
+
+bool t_user::get_speex_dsp_agc(void) const {
+	bool result;
+	mtx_user.lock();
+	result = speex_dsp_agc;
+	mtx_user.unlock();
+	return result;
+}
+
+unsigned short t_user::get_speex_dsp_agc_level(void) const {
+	unsigned short result;
+	mtx_user.lock();
+	result = speex_dsp_agc_level;
+	mtx_user.unlock();
+	return result;
+}
+
+bool t_user::get_speex_dsp_aec(void) const {
+	bool result;
+	mtx_user.lock();
+	result = speex_dsp_aec;
+	mtx_user.unlock();
+	return result;
+}
+
+bool t_user::get_speex_dsp_nrd(void) const {
+	bool result;
+	mtx_user.lock();
+	result = speex_dsp_nrd;
+	mtx_user.unlock();
+	return result;
+}
+
 unsigned short t_user::get_ilbc_payload_type(void) const {
 	unsigned short result;
 	mtx_user.lock();
@@ -1000,51 +1091,38 @@ bool t_user::get_ext_replaces(void) const {
 }
 
 bool t_user::get_referee_hold(void) const {
-	bool result;
-	mtx_user.lock();
-	result = referee_hold;
-	mtx_user.unlock();
-	return result;
+	t_mutex_guard guard(mtx_user);
+	return referee_hold;
 }
 
 bool t_user::get_referrer_hold(void) const {
-	bool result;
-	mtx_user.lock();
-	result = referrer_hold;
-	mtx_user.unlock();
-	return result;
+	t_mutex_guard guard(mtx_user);
+	return referrer_hold;
 }
 
 bool t_user::get_allow_refer(void) const {
-	bool result;
-	mtx_user.lock();
-	result = allow_refer;
-	mtx_user.unlock();
-	return result;
+	t_mutex_guard guard(mtx_user);
+	return allow_refer;
 }
 
 bool t_user::get_ask_user_to_refer(void) const {
-	bool result;
-	mtx_user.lock();
-	result = ask_user_to_refer;
-	mtx_user.unlock();
-	return result;
+	t_mutex_guard guard(mtx_user);
+	return ask_user_to_refer;
 }
 
 bool t_user::get_auto_refresh_refer_sub(void) const {
-	bool result;
-	mtx_user.lock();
-	result = auto_refresh_refer_sub;
-	mtx_user.unlock();
-	return result;
+	t_mutex_guard guard(mtx_user);
+	return auto_refresh_refer_sub;
 }
 
 bool t_user::get_attended_refer_to_aor(void) const {
-	bool result;
-	mtx_user.lock();
-	result = attended_refer_to_aor;
-	mtx_user.unlock();
-	return result;
+	t_mutex_guard guard(mtx_user);
+	return attended_refer_to_aor;
+}
+
+bool t_user::get_allow_transfer_consultation_inprog(void) const {
+	t_mutex_guard guard(mtx_user);
+	return allow_transfer_consultation_inprog;
 }
 
 bool t_user::get_send_p_preferred_id(void) const {
@@ -1097,6 +1175,16 @@ t_url t_user::get_stun_server(void) const {
 	return result;
 }
 
+bool t_user::get_persistent_tcp(void) const {
+	t_mutex_guard guard(mtx_user);
+	return persistent_tcp;
+}
+
+bool t_user::get_enable_nat_keepalive(void) const {
+	t_mutex_guard guard(mtx_user);
+	return enable_nat_keepalive;
+}
+
 unsigned short t_user::get_timer_noanswer(void) const {
 	unsigned short result;
 	mtx_user.lock();
@@ -1105,13 +1193,18 @@ unsigned short t_user::get_timer_noanswer(void) const {
 	return result;
 }
 
-unsigned long t_user::get_timer_nat_keepalive(void) const {
+unsigned short t_user::get_timer_nat_keepalive(void) const {
 	unsigned short result;
 	mtx_user.lock();
 	result = timer_nat_keepalive;
 	mtx_user.unlock();
 	return result;
 }
+
+unsigned short t_user::get_timer_tcp_ping(void) const {
+	t_mutex_guard guard(mtx_user);
+	return timer_tcp_ping;
+}
  
 bool t_user::get_display_useronly_phone(void) const {
 	bool result;
@@ -1145,6 +1238,11 @@ string t_user::get_special_phone_symbols(void) const {
 	return result;
 }
 
+bool t_user::get_use_tel_uri_for_phone(void) const {
+	t_mutex_guard guard(mtx_user);
+	return use_tel_uri_for_phone;
+}
+
 string t_user::get_ringtone_file(void) const {
 	string result;
 	mtx_user.lock();
@@ -1321,6 +1419,11 @@ unsigned short t_user::get_im_max_sessions(void) const {
 	return result;
 }
 
+bool t_user::get_im_send_iscomposing(void) const {
+	t_mutex_guard guard(mtx_user);
+	return im_send_iscomposing;
+}
+
 unsigned long t_user::get_pres_subscription_time(void) const {
 	unsigned long result;
 	mtx_user.lock();
@@ -1388,6 +1491,16 @@ void t_user::set_auth_pass(const string &pass) {
 	mtx_user.unlock();
 }
 
+void t_user::set_auth_aka_op(const uint8 *aka_op) {
+	t_mutex_guard guard(mtx_user);
+	memcpy(auth_aka_op, aka_op, AKA_OPLEN);
+}
+
+void t_user::set_auth_aka_amf(const uint8 *aka_amf) {
+	t_mutex_guard guard(mtx_user);
+	memcpy(auth_aka_amf, aka_amf, AKA_AMFLEN);
+}
+
 void t_user::set_use_outbound_proxy(bool b) {
 	mtx_user.lock();
 	use_outbound_proxy = b;
@@ -1508,12 +1621,6 @@ void t_user::set_speex_abr_wb(int abr) {
 	mtx_user.unlock();
 }
 
-void t_user::set_speex_vad(bool b) {
-	mtx_user.lock();
-	speex_vad = b;
-	mtx_user.unlock();
-}
-
 void t_user::set_speex_dtx(bool b) {
 	mtx_user.lock();
 	speex_dtx = b;
@@ -1526,12 +1633,48 @@ void t_user::set_speex_penh(bool b) {
 	mtx_user.unlock();
 }
 
+void t_user::set_speex_quality(unsigned short quality) {
+	mtx_user.lock();
+	speex_quality = quality;
+	mtx_user.unlock();
+}
+
 void t_user::set_speex_complexity(unsigned short complexity) {
 	mtx_user.lock();
 	speex_complexity = complexity;
 	mtx_user.unlock();
 }
 
+void t_user::set_speex_dsp_vad(bool b) {
+	mtx_user.lock();
+	speex_dsp_vad = b;
+	mtx_user.unlock();
+}
+
+void t_user::set_speex_dsp_agc(bool b) {
+	mtx_user.lock();
+	speex_dsp_agc = b;
+	mtx_user.unlock();
+}
+
+void t_user::set_speex_dsp_agc_level(unsigned short level) {
+	mtx_user.lock();
+	speex_dsp_agc_level = level;
+	mtx_user.unlock();
+}
+
+void t_user::set_speex_dsp_aec(bool b) {
+	mtx_user.lock();
+	speex_dsp_aec = b;
+	mtx_user.unlock();
+}
+
+void t_user::set_speex_dsp_nrd(bool b) {
+	mtx_user.lock();
+	speex_dsp_nrd = b;
+	mtx_user.unlock();
+}
+
 void t_user::set_ilbc_payload_type(unsigned short payload_type) {
 	mtx_user.lock();
 	ilbc_payload_type = payload_type;
@@ -1683,9 +1826,8 @@ void t_user::set_ext_replaces(bool b) {
 }
 
 void t_user::set_referee_hold(bool b) {
-	mtx_user.lock();
+	t_mutex_guard guard(mtx_user);
 	referee_hold = b;
-	mtx_user.unlock();
 }
 
 void t_user::set_referrer_hold(bool b) {
@@ -1695,27 +1837,28 @@ void t_user::set_referrer_hold(bool b) {
 }
 
 void t_user::set_allow_refer(bool b) {
-	mtx_user.lock();
+	t_mutex_guard guard(mtx_user);
 	allow_refer = b;
-	mtx_user.unlock();
 }
 
 void t_user::set_ask_user_to_refer(bool b) {
-	mtx_user.lock();
+	t_mutex_guard guard(mtx_user);
 	ask_user_to_refer = b;
-	mtx_user.unlock();
 }
 
 void t_user::set_auto_refresh_refer_sub(bool b) {
-	mtx_user.lock();
+	t_mutex_guard guard(mtx_user);
 	auto_refresh_refer_sub = b;
-	mtx_user.unlock();
 }
 
 void t_user::set_attended_refer_to_aor(bool b) {
-	mtx_user.lock();
+	t_mutex_guard guard(mtx_user);
 	attended_refer_to_aor = b;
-	mtx_user.unlock();
+}
+
+void t_user::set_allow_transfer_consultation_inprog(bool b) {
+	t_mutex_guard guard(mtx_user);
+	allow_transfer_consultation_inprog = b;
 }
 
 void t_user::set_send_p_preferred_id(bool b) {
@@ -1758,6 +1901,16 @@ void t_user::set_stun_server(const t_url &url) {
 	mtx_user.unlock();
 }
 
+void t_user::set_persistent_tcp(bool b) {
+	t_mutex_guard guard(mtx_user);
+	persistent_tcp = b;
+}
+
+void t_user::set_enable_nat_keepalive(bool b) {
+	t_mutex_guard guard(mtx_user);
+	enable_nat_keepalive = b;
+}
+
 void t_user::set_timer_noanswer(unsigned short timer) {
 	mtx_user.lock();
 	timer_noanswer = timer;
@@ -1770,6 +1923,11 @@ void t_user::set_timer_nat_keepalive(unsigned short timer) {
 	mtx_user.unlock();
 }
 
+void t_user::set_timer_tcp_ping(unsigned short timer) {
+	t_mutex_guard guard(mtx_user);
+	timer_tcp_ping = timer;
+}
+
 void t_user::set_display_useronly_phone(bool b) {
 	mtx_user.lock();
 	display_useronly_phone = b;
@@ -1794,6 +1952,11 @@ void t_user::set_special_phone_symbols(const string &symbols) {
 	mtx_user.unlock();
 }
 
+void t_user::set_use_tel_uri_for_phone(bool b) {
+	t_mutex_guard guard(mtx_user);
+	use_tel_uri_for_phone = b;
+}
+
 void t_user::set_ringtone_file(const string &file) {
 	mtx_user.lock();
 	ringtone_file = file;
@@ -1926,6 +2089,11 @@ void t_user::set_im_max_sessions(unsigned short max_sessions) {
 	mtx_user.unlock();
 }
 
+void t_user::set_im_send_iscomposing(bool b) {
+	t_mutex_guard guard(mtx_user);
+	im_send_iscomposing = b;
+}
+
 void t_user::set_pres_subscription_time(unsigned long t) {
 	mtx_user.lock();
 	pres_subscription_time = t;
@@ -2047,6 +2215,10 @@ bool t_user::read_config(const string &filename, string &error_msg) {
 			auth_name = value;
 		} else if (parameter == FLD_AUTH_PASS) {
 			auth_pass = value;
+		} else if (parameter == FLD_AUTH_AKA_OP) {
+			hex2binary(value, auth_aka_op);
+		} else if (parameter == FLD_AUTH_AKA_AMF) {
+			hex2binary(value, auth_aka_amf);
 		} else if (parameter == FLD_CODECS) {
 			vector<string> l = split(value, ',');
 			if (l.size() > 0) codecs.clear();
@@ -2139,6 +2311,8 @@ bool t_user::read_config(const string &filename, string &error_msg) {
 			auto_refresh_refer_sub = yesno2bool(value);
 		} else if (parameter == FLD_ATTENDED_REFER_TO_AOR) {
 			attended_refer_to_aor = yesno2bool(value);
+		} else if (parameter == FLD_ALLOW_XFER_CONSULT_INPROG) {
+			allow_transfer_consultation_inprog = yesno2bool(value);
 		} else if (parameter == FLD_SEND_P_PREFERRED_ID) {
 			send_p_preferred_id = yesno2bool(value);
 		} else if (parameter == FLD_SIP_TRANSPORT) {
@@ -2151,10 +2325,16 @@ bool t_user::read_config(const string &filename, string &error_msg) {
 			nat_public_ip = value;
 		} else if (parameter == FLD_STUN_SERVER) {
 			use_stun = set_server_value(stun_server, "stun", value);
+		} else if (parameter == FLD_PERSISTENT_TCP) {
+			persistent_tcp = yesno2bool(value);
+		} else if (parameter == FLD_ENABLE_NAT_KEEPALIVE) {
+			enable_nat_keepalive = yesno2bool(value);
 		} else if (parameter == FLD_TIMER_NOANSWER) {
 			timer_noanswer = atoi(value.c_str());
 		} else if (parameter == FLD_TIMER_NAT_KEEPALIVE) {
 			timer_nat_keepalive = atoi(value.c_str());
+		} else if (parameter == FLD_TIMER_TCP_PING) {
+			timer_tcp_ping = atoi(value.c_str());
 		} else if (parameter == FLD_EXT_100REL) {
 			ext_100rel = str2ext_support(value);
 			if (ext_100rel == EXT_INVALID) {
@@ -2197,12 +2377,23 @@ bool t_user::read_config(const string &filename, string &error_msg) {
 			speex_abr_nb = atoi(value.c_str());
 		} else if (parameter == FLD_SPEEX_ABR_WB) {
 			speex_abr_wb = atoi(value.c_str());
-		} else if (parameter == FLD_SPEEX_VAD) {
-			speex_vad = yesno2bool(value);
 		} else if (parameter == FLD_SPEEX_DTX) {
 			speex_dtx = yesno2bool(value);
 		} else if (parameter == FLD_SPEEX_PENH) {
 			speex_penh = yesno2bool(value);
+		} else if (parameter == FLD_SPEEX_QUALITY) {
+			speex_quality = atoi(value.c_str());
+			if (speex_quality > 10) {
+				error_msg = "Syntax error in file ";
+				error_msg += f;
+				error_msg += "\n";
+				error_msg += "Invalid value for speex quality: ";
+				error_msg += value;
+				log_file->write_report(error_msg, "t_user::read_config",
+					LOG_NORMAL, LOG_CRITICAL);
+				mtx_user.unlock();
+				return false;	
+			}
 		} else if (parameter == FLD_SPEEX_COMPLEXITY) {
 			speex_complexity = atoi(value.c_str());
 			if (speex_complexity < 1 || speex_complexity > 10) {
@@ -2216,6 +2407,27 @@ bool t_user::read_config(const string &filename, string &error_msg) {
 				mtx_user.unlock();
 				return false;	
 			}
+		} else if (parameter == FLD_SPEEX_DSP_VAD) {
+			speex_dsp_vad = yesno2bool(value);
+		} else if (parameter == FLD_SPEEX_DSP_AGC) {
+			speex_dsp_agc = yesno2bool(value);
+		} else if (parameter == FLD_SPEEX_DSP_AEC) {
+			speex_dsp_aec = yesno2bool(value);
+		} else if (parameter == FLD_SPEEX_DSP_NRD) {
+			speex_dsp_nrd = yesno2bool(value);
+		} else if (parameter == FLD_SPEEX_DSP_AGC_LEVEL) {
+			speex_dsp_agc_level = atoi(value.c_str());
+			if (speex_dsp_agc_level < 1 || speex_dsp_agc_level > 100) {
+				error_msg = "Syntax error in file ";
+				error_msg += f;
+				error_msg += "\n";
+				error_msg += "Invalid value for automatic gain control level: ";
+				error_msg += value;
+				log_file->write_report(error_msg, "t_user::read_config",
+					LOG_NORMAL, LOG_CRITICAL);
+				mtx_user.unlock();
+				return false;	
+			}
 		} else if (parameter == FLD_ILBC_PAYLOAD_TYPE) {
 			ilbc_payload_type = atoi(value.c_str());
 		} else if (parameter == FLD_ILBC_MODE) {
@@ -2248,6 +2460,8 @@ bool t_user::read_config(const string &filename, string &error_msg) {
 			remove_special_phone_symbols = yesno2bool(value);
 		} else if (parameter == FLD_SPECIAL_PHONE_SYMBOLS) {
 			special_phone_symbols = value;
+		} else if (parameter == FLD_USE_TEL_URI_FOR_PHONE) {
+			use_tel_uri_for_phone = yesno2bool(value);
 		} else if (parameter == FLD_USER_RINGTONE_FILE) {
 			ringtone_file = value;
 		} else if (parameter == FLD_USER_RINGBACK_FILE) {
@@ -2295,6 +2509,8 @@ bool t_user::read_config(const string &filename, string &error_msg) {
 			mwi_vm_address = value;
 		} else if (parameter == FLD_IM_MAX_SESSIONS) {
 			im_max_sessions = atoi(value.c_str());
+		} else if (parameter == FLD_IM_SEND_ISCOMPOSING) {
+			im_send_iscomposing = yesno2bool(value);
 		} else if (parameter == FLD_PRES_SUBSCRIPTION_TIME) {
 			pres_subscription_time = atol(value.c_str());
 		} else if (parameter == FLD_PRES_PUBLICATION_TIME) {
@@ -2382,6 +2598,8 @@ bool t_user::write_config(const string &filename, string &error_msg) {
 	config << FLD_AUTH_REALM << '=' << auth_realm << endl;
 	config << FLD_AUTH_NAME << '=' << auth_name << endl;
 	config << FLD_AUTH_PASS << '=' << auth_pass << endl;
+	config << FLD_AUTH_AKA_OP << '=' << binary2hex(auth_aka_op, AKA_OPLEN) << endl;
+	config << FLD_AUTH_AKA_AMF << '=' << binary2hex(auth_aka_amf, AKA_AMFLEN) << endl;
 	config << endl;
 
 	// Write SIP SERVER settings
@@ -2466,10 +2684,15 @@ bool t_user::write_config(const string &filename, string &error_msg) {
 	// config << FLD_SPEEX_ABR_NB << '=' << speex_abr_nb << endl;
 	// config << FLD_SPEEX_ABR_WB << '=' << speex_abr_wb << endl;
 	config << bit_rate_type2str(speex_bit_rate_type) << endl;
-	config << FLD_SPEEX_VAD << '=' << bool2yesno(speex_vad) << endl;
 	config << FLD_SPEEX_DTX << '=' << bool2yesno(speex_dtx) << endl;
 	config << FLD_SPEEX_PENH << '=' << bool2yesno(speex_penh) << endl;
+	config << FLD_SPEEX_QUALITY << '=' << speex_quality << endl;
 	config << FLD_SPEEX_COMPLEXITY << '=' << speex_complexity << endl;
+	config << FLD_SPEEX_DSP_VAD << '=' << bool2yesno(speex_dsp_vad) << endl;
+	config << FLD_SPEEX_DSP_AGC << '=' << bool2yesno(speex_dsp_agc) << endl;
+	config << FLD_SPEEX_DSP_AEC << '=' << bool2yesno(speex_dsp_aec) << endl;
+	config << FLD_SPEEX_DSP_NRD << '=' << bool2yesno(speex_dsp_nrd) << endl;
+	config << FLD_SPEEX_DSP_AGC_LEVEL << '=' << speex_dsp_agc_level << endl;
 	config << FLD_ILBC_PAYLOAD_TYPE << '=' << ilbc_payload_type << endl;
 	config << FLD_ILBC_MODE << '=' << ilbc_mode << endl;
 	config << FLD_G726_16_PAYLOAD_TYPE << '=' << g726_16_payload_type << endl;
@@ -2526,6 +2749,8 @@ bool t_user::write_config(const string &filename, string &error_msg) {
 	config << bool2yesno(auto_refresh_refer_sub) << endl;
 	config << FLD_ATTENDED_REFER_TO_AOR << '=';
 	config << bool2yesno(attended_refer_to_aor) << endl;
+	config << FLD_ALLOW_XFER_CONSULT_INPROG << '=';
+	config << bool2yesno(allow_transfer_consultation_inprog) << endl;
 	config << FLD_SEND_P_PREFERRED_ID << '=';
 	config << bool2yesno(send_p_preferred_id) << endl;
 	config << endl;
@@ -2545,12 +2770,15 @@ bool t_user::write_config(const string &filename, string &error_msg) {
 	} else {
 		config << FLD_STUN_SERVER << '=' << endl;
 	}
+	config << FLD_PERSISTENT_TCP << '=' << bool2yesno(persistent_tcp) << endl;
+	config << FLD_ENABLE_NAT_KEEPALIVE << '=' << bool2yesno(enable_nat_keepalive) << endl;
 	config << endl;
 
 	// Write TIMER settings
 	config << "# TIMERS\n";
 	config << FLD_TIMER_NOANSWER << '=' << timer_noanswer << endl;
 	config << FLD_TIMER_NAT_KEEPALIVE << '=' << timer_nat_keepalive << endl;
+	config << FLD_TIMER_TCP_PING << '=' << timer_tcp_ping << endl;
 	config << endl;
 
 	// Write ADDRESS FORMAT settings
@@ -2562,6 +2790,7 @@ bool t_user::write_config(const string &filename, string &error_msg) {
 	config << FLD_REMOVE_SPECIAL_PHONE_SYM << '=';
 	config << bool2yesno(remove_special_phone_symbols) << endl;
 	config << FLD_SPECIAL_PHONE_SYMBOLS << '=' << special_phone_symbols << endl;
+	config << FLD_USE_TEL_URI_FOR_PHONE << '=' << bool2yesno(use_tel_uri_for_phone) << endl;
 	config << endl;
 	
 	// Write RING TONE settings
@@ -2620,6 +2849,7 @@ bool t_user::write_config(const string &filename, string &error_msg) {
 	
 	config << "# INSTANT MESSAGE\n";
 	config << FLD_IM_MAX_SESSIONS << '=' << im_max_sessions << endl;
+	config << FLD_IM_SEND_ISCOMPOSING << '=' << bool2yesno(im_send_iscomposing) << endl;
 	config << endl;
 	
 	// Write presence settings
@@ -2661,10 +2891,15 @@ string t_user::get_filename(void) const {
 	return result;
 }
 
-void t_user::set_config(string filename) {
-	mtx_user.lock();
+bool t_user::set_config(string filename) {
+	t_mutex_guard guard(mtx_user);
+	
+	struct stat stat_buf;
+
 	config_filename = filename;
-	mtx_user.unlock();
+	string fullpath = expand_filename(filename);
+	
+	return (stat(fullpath.c_str(), &stat_buf) != 0);
 }
 
 string t_user::get_profile_name(void) const {
@@ -2710,7 +2945,7 @@ string t_user::get_contact_name(void) const {
 	s += '_';
 	
 	// Cut of port and/or uri-parameters if present in domain
-	int i = domain.find_first_of(":;");
+	string::size_type i = domain.find_first_of(":;");
 	if (i != string::npos) {
 		// Some broken SIP proxies think that their own address appears
 		// in the contact header when they see the domain in the user part.
@@ -2806,6 +3041,8 @@ string t_user::create_user_contact(bool anonymous, const string &auto_ip) {
 		case SIP_TRANS_TCP:
 			s += ";transport=tcp";
 			break;
+		default:
+			break;
 		}
 	}
 
@@ -2898,3 +3135,13 @@ t_url t_user::get_mwi_uri(void) const {
 	
 	return u;
 }
+
+bool t_user::is_diamondcard_account(void) const {
+	// A profile is a Diamondcard account if the end configured domain
+	// is equal to the DIAMONDCARD_DOMAIN
+	size_t domain_len = strlen(DIAMONDCARD_DOMAIN);
+	if (domain.size() < domain_len) return false;
+	
+	size_t pos = domain.size() - domain_len;
+	return (domain.substr(pos) == DIAMONDCARD_DOMAIN);
+}
